import audio, explosion, graphic, gunfire, player, math

#---------- Green Disk ----------#
# Zig-zags towards the enemy     #
#--------------------------------#
class Green_Disk:
    def __init__(self, location):
        self.action     = [self.state0, self.state1, self.state2, self.state3, self.state4]
        self.dead       = False
        self.hitpoints  = 7
        self.image      = 0
        self.location   = location
        self.state      = 0
        self.timer      = 0
        self.size       = 13
    
    def damage(self, quantity):
        self.hitpoints -= quantity
        if self.hitpoints <= 0:
            self.dead = True
            player.score += 30
            explosion.create(self.location, 0)
    
    def draw(self):
        global images
        graphic.draw_centered(images[self.image], graphic.screen, self.location)
    
    def state0(self):
        self.location = (self.location[0]-2, self.location[1]+1)
        if self.timer == 50:
            audio.zap.play()
            gunfire.create(self.location, (-7, 4), (0,150,0))
            self.timer = 0
            self.state = 1
        if self.location[0] < player.location[0]:
            self.timer = 0
            self.state = 3

    def state1(self):
        if self.timer == 20:
            self.timer = 0
            self.state = 2
    
    def state2(self):
        self.location = (self.location[0]-1, self.location[1]+2)
        if self.timer == 30:
            self.timer = 0
            self.state = 0
        if self.location[0] < player.location[0]:
            self.timer = 0
            self.state = 3
    
    def state3(self):
        self.target = (player.location[0]+60, player.location[1]-40)
        self.state = 4

        if self.timer%50 == 0:
            audio.zap.play()
            gunfire.create(self.location, (-7, 4), (0,150,0))

    def state4(self):
        xspeed = self.location[0] - self.target[0]
        yspeed = self.location[1] - self.target[1]
        if xspeed > 2:
            xspeed = 2
        elif xspeed < -2:
            xspeed = -2
        if yspeed > 2:
            yspeed = 2
        elif yspeed < -2:
            yspeed = -2
        
        self.location = (self.location[0]-xspeed, self.location[1]-yspeed)

        if self.timer%50 == 0:
            audio.zap.play()
            gunfire.create(self.location, (-7, 4), (0,150,0))

        if xspeed == 0 and yspeed == 0:
            self.state = 3
    
    def update(self):
        self.timer+=1
        self.action[self.state]()

#---------- Yellow Disk ---------#
# Moves up and down while firing #
#--------------------------------#
class Yellow_Disk:
    def __init__(self, location):
        self.action     = [self.state0, self.state1, self.state2, self.state3]
        self.dead       = False
        self.firerate   = 14
        self.hitpoints  = 23
        self.image      = 1
        self.location   = location
        self.state      = 0
        self.timer      = 0
        self.size       = 15

    def damage(self, quantity):
        self.hitpoints -= quantity
        if self.hitpoints <= 0:
            self.dead = True
            player.score += 150
            explosion.create(self.location, 0)
            explosion.create((self.location[0]-5, self.location[1]-1), -7)
            explosion.create((self.location[0]+4, self.location[1]-3), -15)
    
    def draw(self):
        global images
        graphic.draw_centered(images[self.image], graphic.screen, self.location)
    
    def state0(self):
        xspeed = player.location[0] - self.location[0] + 50
        if xspeed > 1:
            xspeed = 1
        elif xspeed < -1:
            xspeed = -1
        if self.timer%self.firerate == 0:
            audio.zap.play()
            gunfire.create(self.location, (-9, 6), (0,150,0))

        if self.location[1] < 380:
            self.location = (self.location[0]+xspeed, self.location[1]+5)
        else:
            self.timer = 0
            self.state = 1
    
    def state1(self):
        if self.timer == 45:
            self.timer = 0
            self.state = 2
    
    def state2(self):
        xspeed = player.location[0] - self.location[0] + 50
        if xspeed > 1:
            xspeed = 1
        elif xspeed < -1:
            xspeed = -1
        if self.timer%self.firerate == 0:
            audio.zap.play()
            gunfire.create(self.location, (-9, 6), (0,150,0))

        if self.location[1] > 0:
            self.location = (self.location[0]+xspeed, self.location[1]-5)
        else:
            self.timer = 0
            self.state = 3


    def state3(self):
        if self.timer == 45:
            self.timer = 0
            self.state = 0

    def update(self):
        self.timer+=1
        self.action[self.state]()

#---------- Purple Disk ---------#
#        Has improved aim        #
#--------------------------------#
class Purple_Disk:
    def __init__(self, location):
        self.action     = [self.state0, self.state1]
        self.dead       = False
        self.firerate   = 50
        self.hitpoints  = 7
        self.image      = 2
        self.location   = location
        self.state      = 0
        self.timer      = 0
        self.size       = 13

    def damage(self, quantity):
        self.hitpoints -= quantity
        if self.hitpoints <= 0:
            self.dead = True
            player.score += 100
            explosion.create(self.location, 0)

    def draw(self):
        global images
        graphic.draw_centered(images[self.image], graphic.screen, self.location)
    
    def state0(self):
        xspeed = player.location[0] - self.location[0] + 60
        yspeed = player.location[1] - self.location[1] - 30
        if xspeed > 2:
            xspeed = 2
        elif xspeed < -2:
            xspeed = -2
        if yspeed > 2:
            yspeed = 2
        elif yspeed < -2:
            yspeed = -2
        self.location = (self.location[0]+xspeed, self.location[1]+yspeed)
        
        if self.timer == self.firerate:
            angle =  math.degrees(math.atan2(self.location[1]-player.location[1], player.location[0]-self.location[0]))
            self.timer = 0
            if angle < 0:
                angle+= 360
            if 190 < angle < 260:
                audio.zap.play()
                gunfire.create(self.location, (math.floor(10.0*math.cos(math.radians(angle))), -math.floor(10.0*math.sin(math.radians(angle)))), (0,150,0))
                self.state = 1
            else:
                self.timer = self.firerate-1
    
    def state1(self):
        if self.timer == self.firerate:
            self.state = 0
            self.timer = 0
            
    def update(self):
        self.timer+=1
        self.action[self.state]()

#----------- Red Disk -----------#
#  Fires in several directions   #
#--------------------------------#
class Red_Disk:
    def __init__(self, location):
        self.action     = [self.state0]
        self.dead       = False
        self.firerate   = 35
        self.hitpoints  = 33
        self.image      = 3
        self.location   = location
        self.state      = 0
        self.timer      = 0
        self.size       = 17

    def damage(self, quantity):
        self.hitpoints -= quantity
        if self.hitpoints <= 0:
            self.dead = True
            player.score += 1000
            explosion.create(self.location, 0)
            explosion.create((self.location[0]-5, self.location[1]-1), -7)
            explosion.create((self.location[0]+4, self.location[1]-3), -15)
            explosion.create((self.location[0], self.location[1]+1), -21)
            explosion.create((self.location[0]-3, self.location[1]-4), -28)
            explosion.create((self.location[0]+4, self.location[1]), -35)

    def draw(self):
        global images
        graphic.draw_centered(images[self.image], graphic.screen, self.location)
    
    def state0(self):
        xspeed = player.location[0] - self.location[0] + 80
        yspeed = player.location[1] - self.location[1] - 40
        if xspeed > 3:
            xspeed = 3
        elif xspeed < -3:
            xspeed = -3
        if yspeed > 3:
            yspeed = 3
        elif yspeed < -3:
            yspeed = -3
        self.location = (self.location[0]+xspeed, self.location[1]+yspeed)
        
        if self.timer%self.firerate == 0:
            audio.zap.play()
            gunfire.create(self.location, (-12, 4), (0,150,0))
            gunfire.create(self.location, (-10, 6), (0,150,0))
            gunfire.create(self.location, (-8, 8), (0,150,0))
            
    def update(self):
        self.timer+=1
        self.action[self.state]()

#---------- Mothership ----------#
# Extremely resilient, good aim. #
#--------------------------------#
class Mothership:
    def __init__(self, location):
        self.action     = [self.state0, self.state1]
        self.dead       = False
        self.firerate   = 70
        self.hitpoints  = 50
        self.image      = 4
        self.location   = location
        self.state      = 0
        self.timer      = 0
        self.size       = 18

    def damage(self, quantity):
        self.hitpoints -= quantity
        if self.hitpoints <= 0:
            self.dead = True
            player.score += 10000
            explosion.create(self.location, 0)

    def draw(self):
        global images
        graphic.draw_centered(images[self.image], graphic.screen, self.location)
    
    def state0(self):
        xspeed = player.location[0] - self.location[0] + 200
        yspeed = player.location[1] - self.location[1] - 75
        if xspeed > 4:
            xspeed = 4
        elif xspeed < -4:
            xspeed = -4
        if yspeed > 4:
            yspeed = 4
        elif yspeed < -4:
            yspeed = -4
        self.location = (self.location[0]+xspeed, self.location[1]+yspeed)
        
        if self.timer == self.firerate:
            angle =  math.degrees(math.atan2(self.location[1]-player.location[1], player.location[0]-self.location[0]))
            self.timer = 0
            if angle < 0:
                angle+= 360
            if 190 < angle < 250:
                audio.zap.play()
                gunfire.create(self.location, (math.floor(11.5*math.cos(math.radians(angle-20))), -math.floor(11.5*math.sin(math.radians(angle-20)))), (0,150,0))
                gunfire.create(self.location, (math.floor(11.5*math.cos(math.radians(angle))),  -math.floor(11.5*math.sin(math.radians(angle)))), (0,150,0))
                gunfire.create(self.location, (math.floor(11.5*math.cos(math.radians(angle+20))), -math.floor(11.5*math.sin(math.radians(angle+20)))), (0,150,0))
                self.state = 1
            else:
                self.timer = self.firerate-1
    
    def state1(self):
        if self.timer == self.firerate/2:
            self.state = 0
            self.timer = 0
            
    def update(self):
        self.timer+=1
        self.action[self.state]()

#--------------------------#
#----- Game Functions -----#
#--------------------------#

def create(name, position):
    global list, types
    list += [types[name](position)]

def draw():
    global list
    for enemy in list:
        enemy.draw()

def quantity():
    global list
    return len(list)

def update():
    global list
    x = 0

    while x < len(list):
        hits = gunfire.hits(list[x].location, list[x].size)
        if hits > 0:
            list[x].damage(hits)
            if list[x].dead:
                list.pop(x)
            else:
                audio.hit.play()
        else:
            x+=1
    
    for enemy in list:
        enemy.update()

def init():
    global images, list, types

    images     = [graphic.load_image("green_disk.png"),
                  graphic.load_image("yellow_disk.png"),
                  graphic.load_image("purple_disk.png"),
                  graphic.load_image("red_disk.png"),
                  graphic.load_image("mothership.png")]
    list       = []
    types      = {'greendisk' : Green_Disk,
                  'yellowdisk': Yellow_Disk,
                  'purpledisk': Purple_Disk,
                  'reddisk'   : Red_Disk,
                  'mothership': Mothership}