import pygame, os, control

#-------------------------#
# Module public variables #
#-------------------------#
# fonts:  font_standard, font_title_1, font_title_2
# screen: screen_surface, screen_size, screen_center
# images:

#-----------------------------------#
# Standard parameters for functions #
#-----------------------------------#

# drawing  (source_image, target_image, location)
# writing  (source_text,  target_image, location, font, (color))
# transform(source_image, argument)

#--------------------------------#
#------- Drawing functions ------#
#--------------------------------#

def draw(source_image, target_image, location):
    target_image.blit(source_image, location)

def draw_centered(source_image, target_image, location):
    size              = source_image.get_size()
    centered_location = (location[0] - size[0]/2, location[1] - size[1]/2)

    target_image.blit(source_image, centered_location)

def draw_line(target_image, p1, p2, color):
    pygame.draw.line(target_image, color, p1, p2)

#--------------------------------#
#------ Writing functions -------#
#--------------------------------#

def write(source_text, target_image, location, font, color=(255,255,255)):
    global aliasing
    
    source_image = font.render(source_text, aliasing, color)
    target_image.blit(source_image, location)

def write_centered(source_text, target_image, location, font, color=(255,255,255)):
    global aliasing

    source_image = font.render(source_text, aliasing, color)

    size              = source_image.get_size()
    centered_location = (location[0] - size[0]/2, location[1] - size[1]/2)

    draw_centered(source_image, target_image, location)

#--------------------------------#
#----- Transform functions ------#
#--------------------------------#

def rotate(source_image, angle):
    return pygame.transform.rotate(source_image, angle)

def resize(source_image, scale):
    size = source_image.get_size()
    return pygame.transform.scale(source_image, (int(size[0]*scale), int(size[1]*scale)))

#def flip_image(source_image):
#    return pygame.transform.flip(source_image, True, False)

#--------------------------------#
#--- Game-specific functions ----#
#--------------------------------#

def clear_screen():
    global screen
    screen.fill((0,0,0))

def clear_sky():
    global screen
    screen.fill((115,227,251))

def clear_sky_area(rect):
    global screen
    screen.fill((115,227,251), rect)

def fade_in():
    global screen
    for x in xrange(0, 25):
        screen.fill(((115*x)/25, (227*x)/25, (251*x)/25))
        pygame.display.flip()
        control.new_frame()

#--------------------------------#
#------- Other functions --------#
#--------------------------------#

def load_image(file_name):
    return pygame.image.load(os.path.join('images', file_name))

def load_font(file_name, size):
    return pygame.font.Font(os.path.join('fonts', file_name), size)

def new_image(size):
    return pygame.Surface(size)

def copy_image_region(source_image, target_image, area):
    target_image.blit(source_image, (0,0), area)

def print_screen():
    global screen, screenshot
    pygame.image.save(screen, "screenshot"+str(screenshot)+".bmp")
    print "Screenshot",screenshot, "taken"
    screenshot+=1

def init():
    # Fonts
    global font_standard, font_title_1, font_title_2
    # Screen
    global screen, screen_size, screen_center
    # Images
    global explosion
    # Other
    global aliasing, screenshot
    
    pygame.display.init()

    info = pygame.display.Info()
    if info.bytesize < 2:
        aliasing = False
    else:
        aliasing = True

    pygame.display.set_mode((400,400))
    pygame.display.set_caption("The Incredible Car Stunt")

    screen        = pygame.display.get_surface()
    screen_size   = screen.get_size()
    screen_center = (screen_size[0]/2, screen_size[1]/2)
    
    font_standard = load_font("freemono.ttf", 15)
    font_title_1  = load_font("freemono.ttf", 20)
    font_title_2  = load_font("freemono.ttf", 25)

    explosion = [resize(load_image("explosion.png"),0.5),
                 resize(load_image("explosion.png"),0.7),
                 resize(load_image("explosion.png"),0.9),
                 load_image("explosion.png"),
                 resize(load_image("explosion.png"),1.2),
                 resize(load_image("explosion.png"),1.5),
                 resize(load_image("explosion.png"),1.8),
                 resize(load_image("explosion.png"),2.0),
                 resize(load_image("explosion.png"),2.1),
                 resize(load_image("explosion.png"),2.2)]

    screenshot = 0

#def update_screen((rect/[rects])):
update_screen = pygame.display.update